#include "cntl.h"

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/time.h>
#include <unistd.h>


int wait_for_ping_reply(int wait_sec);
int chogwash_wait_for_packet();
int nhogwash_resend_packet();

/***************************************************
* Find the encryption key that goes with this IP
***************************************************/
TargetRecord* chogwash_get_target_from_ip(struct in_addr ip){
	int i;
	
	for (i=0;i<pv.num_targets;i++){
		if (memcmp(&pv.targets[i].ip, &ip, sizeof(struct in_addr))==0){
			return &pv.targets[i];
		} 
	}
	printf("Could resolve ip/key\n");
	return NULL;
}

/*******************************************************
* NonBlocking call to ping remote box
*******************************************************/
int chogwash_send_ping(struct in_addr ip, int wait_sec, int (*pResponseCallback) (int Success, char* Data, int Datalen, void* user)){
	ControlPacketPing	cpp;
	
	cpp.MagicToken1=CONTROL_MAGIC_TOKEN;
	cpp.MagicToken2=CONTROL_MAGIC_TOKEN;
	cpp.PacketType=CONTROL_PACKET_PING;
	sprintf(cpp.PingMessage, "This is a test.");
	
	return nhogwash_send_packet(ip, wait_sec, (char*)&cpp, sizeof(ControlPacketPing), pResponseCallback);
}

/*******************************************************
* This function gets called when we get a ping reply
*******************************************************/
int nhogwash_ping_callback(int Success, char* Data, int DataLen, void* user){
	ControlPacketPingReply*		cppr;
	
	printf("Ping Callback Called\n");

	/*sanity checks*/
	if (DataLen!=sizeof(ControlPacketPingReply)){
		printf("Packet was wrong length\n");
		return;
	}

	cppr=(ControlPacketPingReply*)Data;
	return BUFFER_FLAG_OK;
}


/*******************************************************
* Blocking call to ping remote box
*******************************************************/
int chogwash_ping(struct in_addr ip, int wait_sec){	
	nhogwash_get_buff_stat(ip, wait_sec, 0, nhogwash_ping_callback, NULL);
	return chogwash_wait_for_packet();
}

/*******************************************************
* NonBlocking call to get the status of a buffer
*******************************************************/
int nhogwash_get_buff_stat(struct in_addr ip, int wait_sec,	unsigned int BuffID, int (*pResponseCallback) (int Success, char* Data, int Datalen, void* user),ControlPacketStatReply* stat){
	TargetRecord*				tr;
	ControlPacketStatRequest	cps;
	char						Buff[65536];
	long int					count;
	
	struct sockaddr_in			sender;
	int							size;
	struct timeval				tv, endtime;
	struct timezone				tz;
	ControlPacketStatReply*		cpsr;

	
	cps.MagicToken1=CONTROL_MAGIC_TOKEN;
	cps.MagicToken2=CONTROL_MAGIC_TOKEN;
	cps.PacketType=CONTROL_PACKET_STAT_REQUEST;
	cps.BuffID=BuffID;
	
	nhogwash_send_packet(ip, wait_sec, (char*)&cps, sizeof(ControlPacketStatRequest), pResponseCallback, (void*)stat );	
}

/*******************************************************
* This function gets called when we get a stat reply
*******************************************************/
int nhogwash_get_buff_stat_callback(int Success, char* Data, int DataLen, void* user){
	ControlPacketStatReply*		cpsr;
	
	//printf("Stat Callback Called\n");

	if (!Success) return BUFFER_FLAG_TIMED_OUT;

	/*sanity checks*/
	if (DataLen!=sizeof(ControlPacketStatReply)){
		printf("Packet was wrong length\n");
		return;
	}

	cpsr=(ControlPacketStatReply*)Data;
	memcpy(user, cpsr, sizeof(ControlPacketStatReply));
	
	return BUFFER_FLAG_OK;
}

/*******************************************************
* Blocking call to get the status of a buffer
*******************************************************/
int chogwash_get_buff_stat(struct in_addr ip, int wait_sec,	unsigned int BuffID, ControlPacketStatReply* stat){	
	nhogwash_get_buff_stat(ip, wait_sec, BuffID, nhogwash_get_buff_stat_callback, stat);
	return chogwash_wait_for_packet();
}

/*******************************************************
* NonBlocking call to get a piece of a buffer
*******************************************************/
int	nhogwash_get_buff_chunk(struct in_addr ip, int wait_sec, unsigned int BuffID, unsigned int Offset, unsigned int Len, ControlPacketBuffReply* p, int (*pResponseCallback) (int Success, char* Data, int Datalen, void* user)){
	TargetRecord*				tr;
	ControlPacketBuffRequest	cpb;
	char						Buff[65536];
	struct sockaddr_in			sa;
	long int					count;
	
	struct sockaddr_in			sender;
	int							size;
	struct timeval				tv, endtime;
	struct timezone				tz;
	ControlPacketBuffReply*		cpbr;

	cpb.MagicToken1=CONTROL_MAGIC_TOKEN;
	cpb.MagicToken2=CONTROL_MAGIC_TOKEN;
	cpb.PacketType=CONTROL_PACKET_BUFF_REQUEST;
	cpb.BuffID=BuffID;
	cpb.Offset=Offset;
	cpb.Len=Len;
	cpb.Flags=0;
	
	nhogwash_send_packet(ip, wait_sec, (char*)&cpb, sizeof(ControlPacketBuffRequest), pResponseCallback, (void*)p);		
}

/*******************************************************
* This function gets called when we get a buff chunk 
*******************************************************/
int nhogwash_get_buff_chunk_callback(int Success, char* Data, int DataLen, void* user){
	ControlPacketBuffReply*	cpbr;
		
	//printf("Buff Chunk Callback Called\n");

	if (!Success) return BUFFER_FLAG_TIMED_OUT;

	/*sanity checks*/
	if (DataLen!=sizeof(ControlPacketBuffReply)){
		printf("Packet was wrong length\n");
		return;
	}

	cpbr=(ControlPacketBuffReply*)Data;	
	memcpy(user, Data, sizeof(ControlPacketBuffReply));
	
	return cpbr->Flags;	
}


/*******************************************************
* Blocking call to get a piece of a buffer
*******************************************************/
int	chogwash_get_buff_chunk(struct in_addr ip, int wait_sec, unsigned int BuffID, unsigned int Offset, unsigned int Len, ControlPacketBuffReply* p){
	nhogwash_get_buff_chunk(ip, wait_sec, BuffID, Offset, Len, p, nhogwash_get_buff_chunk_callback);
	return chogwash_wait_for_packet();
}


/*******************************************************
* Blocking call to get an entire buffer
*******************************************************/
int	chogwash_get_buff(struct in_addr ip, int wait_sec,unsigned int BuffID, char* OutBuff, unsigned int OutBuffLen){
	ControlPacketStatReply	st;
	int						i,j;
	int						retries;
	ControlPacketBuffReply	br;
	
	/*get the length of the buffer*/
	if (chogwash_get_buff_stat(ip, 3, BuffID, &st)){
		printf("I Couldn't Stat the buffer\n");
		return FALSE;
	}
	
	retries=0;
	for (i=0;i<=st.Size;i=i+MAX_BUFFER_SEGMENT){
		chogwash_get_buff_chunk(ip, wait_sec, BuffID, i, MAX_BUFFER_SEGMENT, &br);
		switch (br.Flags){
		case BUFFER_FLAG_END_OF_BUFFER:
		case BUFFER_FLAG_OK:
			memcpy(OutBuff+i, br.Data, br.Len);
			retries=0;
			break;
		case BUFFER_FLAG_TIMED_OUT:
			if (retries>BUFFER_MAX_RETRY) return BUFFER_FLAG_TIMED_OUT;
			i=i-MAX_BUFFER_SEGMENT;
			retries++;
			break;
		default:
			printf("Unknown Error %i\n", br.Flags);
			return BUFFER_FLAG_UNKNOWN_ERR;
		}
	}
	
}

/*******************************************************
* Blocking call to set an entire buffer
*******************************************************/
int	chogwash_set_buff(struct in_addr ip, int wait_sec, unsigned int	BuffID,	char* OutBuff,unsigned int OutBuffLen){
	ControlPacketSetBuffer	c;
	int						i;
	
	bzero(&c, sizeof(ControlPacketSetBuffer));
	c.MagicToken1=CONTROL_MAGIC_TOKEN;
	c.MagicToken2=CONTROL_MAGIC_TOKEN;
	c.PacketType=CONTROL_PACKET_SET_BUFF;
	c.PacketID=random()*65535;
	c.BuffID=STOCK_BUFFER_NEW_RULES;
	
	for (i=0;i<OutBuffLen;i=i+MAX_BUFFER_SEGMENT){
		c.Offset=i;
		if ((i+MAX_BUFFER_SEGMENT) < OutBuffLen){
			c.Len=MAX_BUFFER_SEGMENT;
		}else{
			c.Len=OutBuffLen%MAX_BUFFER_SEGMENT;
		}
		memcpy(c.Data, OutBuff+i, c.Len);

		nhogwash_send_packet(ip, wait_sec, (char*)&c, sizeof(ControlPacketSetBuffer), NULL, NULL);		
	}	
}		


int chogwash_send_ping_old(struct in_addr ip){
	TargetRecord*		tr;
	ControlPacketPing	cpp;
	char				Buff[65536];
	struct sockaddr_in	sa;
	long int			count;
	
	tr=chogwash_get_target_from_ip(ip);
	if (!tr) return BUFFER_FLAG_KEY_ERROR;
	
	sa.sin_family=AF_INET;
	sa.sin_port=htons(5757);
	sa.sin_addr = ip;
	bzero(&sa.sin_zero, 8);
	
	cpp.MagicToken1=CONTROL_MAGIC_TOKEN;
	cpp.MagicToken2=CONTROL_MAGIC_TOKEN;
	cpp.PacketType=CONTROL_PACKET_PING;
	sprintf(cpp.PingMessage, "This is a test.");
	
	//memcpy(&Buff, &cpp, sizeof(ControlPacketPing));
	*(unsigned int*)Buff=cpp.MagicToken1;
	count=blockEncrypt(&tr->key.ci, &tr->key.ki, (char*)&cpp+4, (sizeof(ControlPacketPing)-4)*8, Buff+4);
	if (count!=(sizeof(ControlPacketPing)-4)*8){
		printf("Failed to encrypt returned %li\n",count);
	}
	
	sendto(pv.sock_fd, Buff, sizeof(ControlPacketPing), 0, (struct sockaddr*)&sa, sizeof(struct sockaddr));
}

int wait_for_ping_reply(int wait_sec){
	char				Buff[65536];
	int					count;
	struct sockaddr_in	sender;
	int					size;
	struct timeval		tv, endtime;
	struct timezone		tz;
	
	gettimeofday(&endtime, &tz);
	endtime.tv_sec=endtime.tv_sec+wait_sec;
		
	while (1){
		size=sizeof(struct sockaddr);
		count=recvfrom(pv.sock_fd, Buff, 65535, 0, (struct sockaddr*)&sender, &size);
		if (count!=-1){			
			return TRUE;
			break;
		}
		gettimeofday(&tv,&tz);
		if (tv.tv_sec>endtime.tv_sec) return FALSE;
		usleep(1000);
	}	
}

/**************************************************
* Send off a packet
**************************************************/
int nhogwash_send_packet(
	struct in_addr	ip, 
	int Retries, 
	char* Data, 
	int DataSize,
	int (*pResponseCallback) (int Success, char* Data, int DataLen, void* user),
	void* user)
{
	int 				i;
	TargetRecord*		target;
	int					count;
	char				Buff[65536];
	struct sockaddr_in	sa;
	struct timeval		tv;
	struct timezone		tz;
	
	/*set up the variables*/
	sa.sin_family=AF_INET;
	sa.sin_port=htons(5757);
	sa.sin_addr = ip;
	bzero(&sa.sin_zero, 8);
	target=chogwash_get_target_from_ip(ip);
	gettimeofday(&tv,&tz);
	
	/*find a free PacketState*/
	for (i=0;i<MAX_PACKETS_IN_FLIGHT;i++){
		if (ps[i].InUse==FALSE) break;
	}
	if (i==MAX_PACKETS_IN_FLIGHT) return FALSE;
	
	/*fill in the packet*/
	ps[i].InUse=TRUE;
	ps[i].Target=target;
	ps[i].PacketID=rand()*65535*65535;
	ps[i].RetriesLeft=Retries;
	ps[i].pResponseCallback=pResponseCallback;
	ps[i].user=user;
	ps[i].LastTime=tv.tv_sec;
	
	/*encrypt the payload*/
	((ControlPacket*)Buff)->MagicToken1=CONTROL_MAGIC_TOKEN;
	((ControlPacket*)Data)->MagicToken2=CONTROL_MAGIC_TOKEN;
	((ControlPacket*)Data)->PacketID=ps[i].PacketID;
	count=blockEncrypt(&target->key.ci, &target->key.ki, Data+4, (DataSize-4)*8, Buff+4);
	if (count!=((DataSize-4)*8)){
		printf("Failed to encrypt returned %li\n",count);
	}
	
	/*make a backup copy for resends*/
	memcpy(ps[i].RawData, Buff, DataSize);
	ps[i].RawSize=DataSize;

	/*send it off*/
	return (sendto(pv.sock_fd, Buff, DataSize, 0, (struct sockaddr*)&sa, sizeof(struct sockaddr))==-1);
}


/**************************************************
* Check to see if a packet is waiting, then read it
**************************************************/
int nhogwash_read_packet(){
	char						Buff[65536];
	long int					count, recvcount;	
	struct sockaddr_in			sender;
	int							size;
	TargetRecord*				target;
	int							i;
	ControlPacket*				cp;


	/*read in the packet*/
	size=sizeof(struct sockaddr);
	recvcount=recvfrom(pv.sock_fd, Buff, 65535, 0, (struct sockaddr*)&sender, &size);
	if (recvcount==-1) return BUFFER_FLAG_NOT_READY;


	/*decrypt the packet*/
	target=chogwash_get_target_from_ip(sender.sin_addr);
	count=blockDecrypt(&target->key.ci, &target->key.ki, Buff+4, (recvcount-4)*8, Buff+4);
	if (count!=(recvcount-4)*8){
		printf("Failed to decrypt returned %li\n",count);
		return BUFFER_FLAG_KEY_ERROR;
	}
	cp=(ControlPacket*)Buff;
	
	/*find the correct handler*/
	for (i=0;i<MAX_PACKETS_IN_FLIGHT;i++){
		//if (ps[i].InUse) printf("State %i %u vs. %u\n",i,ps[i].PacketID, cp->PacketID);
		if (ps[i].InUse && ps[i].PacketID==cp->PacketID) break;
	}
	if (i==MAX_PACKETS_IN_FLIGHT){ 
		printf("Couldn't find a state for this packet %u\n", cp->PacketID);
		return BUFFER_FLAG_NO_STATE;
	}	
	
	/*call the handler*/
	ps[i].InUse=FALSE;
	if (ps[i].pResponseCallback) return ps[i].pResponseCallback(TRUE, Buff, recvcount, ps[i].user);
}

/**************************************************
* resend any packets that may be lost in transit
**************************************************/
int nhogwash_resend_packet(){
	int				i;
	struct timeval	tv;
	struct timezone	tz;
	struct sockaddr_in	sa;
	int				StuffToDo;
		
	gettimeofday(&tv,&tz);
	
	StuffToDo=FALSE;
	for (i=0;i<MAX_PACKETS_IN_FLIGHT;i++){
		if (ps[i].InUse){
			if ((tv.tv_sec-ps[i].LastTime)>1){
				printf("Packet %i is too old\n", i);
				if (ps[i].RetriesLeft>0){
					printf("Resending packet\n");
					sa.sin_family=AF_INET;
					sa.sin_port=htons(5757);
					sa.sin_addr = ps[i].Target->ip;
					bzero(&sa.sin_zero, 8);
					sendto(pv.sock_fd, ps[i].RawData, ps[i].RawSize, 0, (struct sockaddr*)&sa, sizeof(struct sockaddr));
					ps[i].RetriesLeft--;
					ps[i].LastTime=tv.tv_sec;
					printf("Retries Left %i\n",ps[i].RetriesLeft);
				}else{
					printf("Expiring Packet\n");
					ps[i].InUse=FALSE;
					if (ps[i].pResponseCallback) 
						ps[i].pResponseCallback(FALSE, NULL, 0, ps[i].user);
				}
			}	
			StuffToDo=TRUE;	
		}
	}
	
	return StuffToDo;
}

/**************************************************
* wait for a reply/failure
**************************************************/
int chogwash_wait_for_packet(){
	struct timeval	tv1, tv2;
	struct timezone	tz1, tz2;
	int				result;
	
	gettimeofday(&tv1,&tz1);
	
	while (1){
		result=nhogwash_read_packet();
		if (result==BUFFER_FLAG_OK) return result;
		if (!nhogwash_resend_packet()) return BUFFER_FLAG_TIMED_OUT;
		usleep(1000);
	}	
}

/**************************************************
* Read the next line from the file into Buff
**************************************************/
int chogwash_get_next_line(int fd, char* Buff, int BuffLen){
	int		count;
	char	c;
	int		num_read;
	char*	this;
	
	num_read=0;
	this=Buff;
	while ((count=read(fd, &c, 1))!=-1){
		if (count==0) break;
		*this=c;
		if (c!=0x20){
		 this++;
		 num_read++;
		} 
		if (c==0x0A) break;
		if (num_read==BuffLen) break;
	}
	this--;
	*this=0;
	
	return num_read;
	
}

/**************************************************
* Get the next field out of the string
**************************************************/
int chogwash_get_next_field(char* InBuff, char StartChar, char EndChar, char* OutBuff, int OutBuffLen){
	int		count;
	char	c;
	int		num_read;
	char	*start, *end;
	
	num_read=0;
	start=InBuff;

	/*seek to the start char*/
	if (StartChar!=0){
		while (*start!=0x0 && *start!=StartChar) start++;
		if (*start==0x0) return 0;
		start++;
	}
	
	/*seek to the ending char*/
	end=start+1;
	while (*end!=0x0 && *end!=EndChar){
		num_read++;
		if (num_read==OutBuffLen) break;
		end++;
	}	
	num_read++;
	memcpy(OutBuff, start, num_read);
	
	return num_read;
	
}

/************************************************
* Read in the contents of the config file
************************************************/
int chogwash_read_config(char* config_file){
	int 	fd;
	char	LineBuff[1024];
	char	FieldBuff[1024];
	char	IntBuff[9];
	int		count;
	int		i;
	
	fd = open(config_file, O_RDONLY);
	if (fd==0){
		printf("Couldn't open config file\n");
		return FALSE;
	}
	
	while (chogwash_get_next_line(fd, LineBuff, 1024)){
		if (LineBuff[0]=='#') continue;
		if (LineBuff[0]==0x0A) continue;
		if (LineBuff[0]==0x0) continue;
		
		if (strncasecmp(LineBuff, "target", 6)==0){
			/*read in a twofish key*/
			bzero(FieldBuff,1024);
			count=chogwash_get_next_field(LineBuff, '(',',',FieldBuff, 1024);	
			if (!inet_aton(FieldBuff, &pv.targets[pv.num_targets].ip)) continue;
			bzero(FieldBuff,1024);
			count=chogwash_get_next_field(LineBuff, ',',')',FieldBuff, 1024);	
			if (count!=((KeySize/32)*8)){
				printf("Wrong key size was %i characters expected %i characters\n",count,((KeySize/32)*8));
				continue;
			}	
			for (i=0;i<(KeySize/32);i++){
				memcpy(IntBuff, FieldBuff+(i*8),8);
				IntBuff[9]=0x0;
				pv.targets[pv.num_targets].key.ki.key32[i]=strtol(IntBuff, NULL, 16); 
			}
			reKey(&pv.targets[pv.num_targets].key.ki);
			bzero(FieldBuff,1024);
			count=chogwash_get_next_field(LineBuff, ')',0,FieldBuff, 1024);	
			snprintf(pv.targets[pv.num_targets].name,512,"%s",FieldBuff);
			pv.num_targets++;
		}else{
			printf("Unrecognized Keyword %s\n",LineBuff);
		}
	}
	
	close(fd);
	
	return TRUE;
}

/****************************************************
* Initialize the control channel
* If a NULL is passed, 
*   ./control.conf
*   /etc/hogwash/control.conf
* are tried
****************************************************/
int chogwash_init(char* config_file){
	int i,j;

	bzero(&ps[0],sizeof(struct packet_state)*MAX_PACKETS_IN_FLIGHT);
	bzero(&pv, sizeof(struct panel_vars));	
	pv.sock_fd=socket(AF_INET, SOCK_DGRAM, 0);
	pv.sock_addr.sin_family=AF_INET;
	pv.sock_addr.sin_port=0;
	pv.sock_addr.sin_addr.s_addr = INADDR_ANY;
	bzero(&(pv.sock_addr.sin_zero), 8);
	bind(pv.sock_fd, (struct sockaddr*)&pv.sock_addr, sizeof(struct sockaddr));
	/*make the socket non_blocking*/
	fcntl(pv.sock_fd, F_SETFL, O_NONBLOCK);
	
	for (i=0;i<MAX_TARGETS;i++){
		makeKey(&pv.targets[i].key.ki, DIR_ENCRYPT, KeySize, NULL);
		cipherInit(&pv.targets[i].key.ci, MODE_ECB, NULL);
	}	
	
	chogwash_read_config(config_file);
}

/*************************************************************
* Close down and free everything
*************************************************************/
int chogwash_destroy(){
	close(pv.sock_fd);
}
