#include "cntl2.h"
#include "panel.h"
#include <gtk/gtk.h>

#define VERSION "0.2"

void UpdateStatus(int SensorNum);
void UpdateStatusStart(int SensorNum);

/*Globals for the summary page*/
GtkWidget*	PacketCount[MAX_TARGETS];
GtkWidget*	TCPCount[MAX_TARGETS];
GtkWidget*	UDPCount[MAX_TARGETS];
GtkWidget*	ICMPCount[MAX_TARGETS];
GtkWidget*	ARPCount[MAX_TARGETS];
GtkWidget*	IPV6Count[MAX_TARGETS];
GtkWidget*	IPXCount[MAX_TARGETS];
GtkWidget*	OtherCount[MAX_TARGETS];
GtkWidget*	AlertCount[MAX_TARGETS];
GtkWidget*	LogCount[MAX_TARGETS];
GtkWidget*	DropCount[MAX_TARGETS];

/*Globals for the individual pages*/
GtkWidget*	RulesBox[MAX_TARGETS];
GtkWidget*	AlertList[MAX_TARGETS];

/******************************************************
* Stolen gtk.org
******************************************************/
GtkWidget *xpm_label_box( GtkWidget *parent, gchar *xpm_filename, gchar *label_text ){
    GtkWidget *box1;
    GtkWidget *label;
    GtkWidget *pixmapwid;
    GdkPixmap *pixmap;
    GdkBitmap *mask;
    GtkStyle *style;

    /* Create box for xpm and label */
    box1 = gtk_hbox_new (FALSE, 0);
    gtk_container_set_border_width (GTK_CONTAINER (box1), 2);

    /* Get the style of the button to get the
     * background color. */
    style = gtk_widget_get_style(parent);

    /* Now on to the xpm stuff */
    pixmap = gdk_pixmap_create_from_xpm (parent->window, &mask,
					 &style->bg[GTK_STATE_NORMAL],
					 xpm_filename);
    pixmapwid = gtk_pixmap_new (pixmap, mask);

    /* Create a label for the button */
    label = gtk_label_new (label_text);

    /* Pack the pixmap and label into the box */
    gtk_box_pack_start (GTK_BOX (box1),
			pixmapwid, FALSE, FALSE, 3);

    gtk_box_pack_start (GTK_BOX (box1), label, FALSE, FALSE, 3);

    gtk_widget_show(pixmapwid);
    gtk_widget_show(label);

    return(box1);
}

/*****************************************************
* Handle an update status click
*****************************************************/
void SummaryUpdateClicked(GtkWidget* w, gpointer data){
	int	SensorNum;
	
	SensorNum=(int)data;
	UpdateStatusStart(SensorNum);
}

/*****************************************************
* Gets Called when an rules buffer is ready
*****************************************************/
void GetRulesEnd(int Success, void* data){
	int						SensorNum;
	GtkWidget*				text;
	JobGetBuff*				j;
	
	if (!Success) return;
	
	j=(JobGetBuff*)data;
	SensorNum=(int)j->User;
	
	gtk_text_freeze(GTK_TEXT(RulesBox[SensorNum]));
	text=gtk_text_new(NULL, NULL);
	gtk_text_set_point(GTK_TEXT(RulesBox[SensorNum]), 0);
	gtk_text_forward_delete(GTK_TEXT(RulesBox[SensorNum]),gtk_text_get_length(GTK_TEXT(RulesBox[SensorNum])));
	gtk_text_insert(GTK_TEXT(RulesBox[SensorNum]), NULL, &text->style->black, NULL, j->TargetBuff, -1);
	gtk_text_thaw(GTK_TEXT(RulesBox[SensorNum]));
	
	free(j->TargetBuff);
	free(data);
}

/*****************************************************
* Start a job to update the rules
*****************************************************/
void GetRulesStart(GtkWidget* w, gpointer data){
	int 					SensorNum;
	ControlPacketStatReply	st;
	char*					Target;
		
	/*get the size of the buffer*/
	SensorNum=(int)data;
	if (!hb_get_stat(Targets[SensorNum].IP, STOCK_BUFFER_RULES, &st)){
		printf("Error getting stat!\n");
		return;
	}
	
	/*get the buffer*/
	Target=(char*)calloc(st.Size+2 ,1);
	hn_get_buff(Targets[SensorNum].IP, STOCK_BUFFER_RULES, Target, st.Size, GetRulesEnd, (void*)SensorNum);
}

/*****************************************************
* Parse the Alert file into AlertRecord's
*****************************************************/
int	ParseAlertRecord(char* TextRecord, AlertRecord* r){
	char*	start;
	char*	end;
	char*	this;
	int		Found;
	int		count;
	
	char	TempBuff[10240];

	if (!TextRecord) return 0;
	if (!r) return 0;
	bzero(r,sizeof(AlertRecord));
	
	/*find the start of the record*/
	start=TextRecord;
	Found=FALSE;
	while (!Found){
		while (*start!='[' && *start!=0x00) start++;
		if (*start==0x00) return 0;
		if (memcmp(start, "[**] [", 6)==0) break;
		start++;
	}

	/*find the end of the record*/
	end=start+1;
	Found=FALSE;
	while (!Found){
		while (*end!="\n"[0] && *end!=0x00) end++;
		if (*end==0x00) return 0;
		if (memcmp(end, "\n\n", 2)==0) break;
		end++;
	}
	
	/*fill out the record*/
	this=start+13;
	count=0;
	while (*this!='[' && this<end){
		r->Desc[count]=*this;
		this++;
		count++;
	}
	this=this+5;
	count=0;
	while (*this!=' ' && this<end){
		r->CharTime[count]=*this;
		this++;
		count++;
	}
	this++;
	count=0;
	while (*this!=':' && this<end){
		TempBuff[count]=*this;
		this++;
		count++;
	}
	TempBuff[count]=0;
	inet_aton(TempBuff, &r->SrcIP);
	
	this++;
	count=0;
	while (*this!=' ' && this<end){
		TempBuff[count]=*this;
		this++;
		count++;
	}
	TempBuff[count]=0;
	r->SrcPort=atoi(TempBuff);
	
	this=this+4;
	count=0;
	while (*this!=':' && this<end){
		TempBuff[count]=*this;
		this++;
		count++;
	}
	TempBuff[count]=0;
	inet_aton(TempBuff, &r->DstIP);
	
	this++;
	count=0;
	while (*this!=' ' && this<end){
		TempBuff[count]=*this;
		this++;
		count++;
	}
	TempBuff[count]=0;
	r->DstPort=atoi(TempBuff);

	return (end-TextRecord);
}

/*****************************************************
* Get Called when an alert buffer is ready
*****************************************************/
void GetAlertEnd(int Success, void* data){
	JobGetBuff*	j;
	int						SensorNum;
	GtkWidget*				text;
	int						i;
	AlertRecord				ARec;
	
	char*					Row[4];
	GdkColormap*			colormap;
	GdkColor				rowcolor;
	int						rowlen;
	char*					beginrow;

	
	if (!Success) return;
	
	j=(JobGetBuff*)data;
	SensorNum=(int)j->User;
	
	colormap=gdk_colormap_get_system();
	rowcolor.red=65535;
	rowcolor.blue=45535;
	rowcolor.green=55535;
	gdk_color_alloc(colormap, &rowcolor);
	
	/*fill in the list box*/
	Row[0]=(char*)calloc(1024,1);
	Row[1]=(char*)calloc(1024,1);
	Row[2]=(char*)calloc(1024,1);
	Row[3]=(char*)calloc(1024,1);
	gtk_clist_freeze(GTK_CLIST(AlertList[SensorNum]));
	gtk_clist_clear(GTK_CLIST(AlertList[SensorNum]));
	beginrow=j->TargetBuff;
	i=0;
	
	while ((rowlen=ParseAlertRecord(beginrow, &ARec))){
		beginrow=beginrow+rowlen;
		i++;
		bzero(Row[0],1024);
		bzero(Row[1],1024);
		bzero(Row[2],1024);
		bzero(Row[3],1024);
		sprintf(Row[0],ARec.CharTime);
		sprintf(Row[1],ARec.Desc);
		sprintf(Row[2],"%s:%u",inet_ntoa(ARec.SrcIP),ARec.SrcPort);
		sprintf(Row[3],"%s:%u",inet_ntoa(ARec.DstIP),ARec.DstPort);
		gtk_clist_prepend(GTK_CLIST(AlertList[SensorNum]),Row);
	}
	while (i>0){
		gtk_clist_set_background(GTK_CLIST(AlertList[SensorNum]), i, &rowcolor);
		i=i-3;
	}
	gtk_clist_thaw(GTK_CLIST(AlertList[SensorNum]));
	
	free(Row[0]);
	free(Row[1]);
	free(Row[2]);
	free(Row[3]);
	free(j->TargetBuff);
	free(data);
}

/*****************************************************
* Start a job to update the alerts
*****************************************************/
void GetAlertStart(GtkWidget* w, gpointer data){
	int 					SensorNum;
	ControlPacketStatReply	st;
	char*					Target;
		
	/*get the size of the buffer*/
	SensorNum=(int)data;
	if (!hb_get_stat(Targets[SensorNum].IP, STOCK_BUFFER_ALERT, &st)){
		printf("Error getting stat!\n");
		return;
	}
	
	/*get the buffer*/
	Target=(char*)calloc(st.Size+2 ,1);
	hn_get_buff(Targets[SensorNum].IP, STOCK_BUFFER_ALERT, Target, st.Size, GetAlertEnd, (void*)SensorNum);
}

/*****************************************************
* Create a sensor window
*****************************************************/
GtkWidget* MakeSensorWindow(int SensorNum){
	GtkWidget*	win;
	GtkWidget*	vbox;
	GtkWidget*	vbox2;
	GtkWidget*	buttonbox;
	GtkWidget*	buttonbox2;
	GtkWidget*	updatebutton;
	GtkWidget*	updatebutton2;
	GtkWidget*	storebutton;
	GtkWidget*	notebook;
	GtkWidget*	vscrollbar;
	GtkWidget*	vscrollbar2;
	GtkWidget*	textbox;
	GtkWidget*	textbox2;
	GtkWidget*	ruleslabel;
	GtkWidget*	alertlabel;
	char*		AlertListLabels[4]= {"Time","Description","Source", "Dest"};
	GtkObject*	adj;
	
	notebook=gtk_notebook_new();
	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(notebook), GTK_POS_TOP);
	ruleslabel=gtk_label_new("Rules");
	gtk_widget_show(ruleslabel);
	
	vbox=gtk_vbox_new(FALSE, 0);
	gtk_widget_show(vbox);
	
	buttonbox=gtk_hbox_new(TRUE, 0);
	gtk_widget_show(buttonbox);
	gtk_box_pack_end(GTK_BOX(vbox), buttonbox, FALSE, FALSE, 0);
	
	textbox=gtk_hbox_new(FALSE, 0);
	gtk_widget_show(textbox);
	gtk_box_pack_end(GTK_BOX(vbox), textbox, TRUE, TRUE, 0);

	RulesBox[SensorNum]=gtk_text_new(NULL, NULL);
	gtk_widget_show(RulesBox[SensorNum]);
	gtk_box_pack_start(GTK_BOX(textbox), RulesBox[SensorNum], TRUE, TRUE, 0);
	
	vscrollbar=gtk_vscrollbar_new(GTK_TEXT(RulesBox[SensorNum])->vadj);
	gtk_widget_show(vscrollbar);
	gtk_box_pack_end(GTK_BOX(textbox), vscrollbar, FALSE, FALSE, 0);
	
	updatebutton=gtk_button_new_with_label("Fetch");
	gtk_widget_show(updatebutton);
	gtk_box_pack_start(GTK_BOX(buttonbox), updatebutton, FALSE, FALSE, 0);
	gtk_signal_connect(GTK_OBJECT(updatebutton), "clicked", GTK_SIGNAL_FUNC(GetRulesStart), (void*)SensorNum);
	gtk_widget_set_usize(updatebutton, 100, 20);
	
	storebutton=gtk_button_new_with_label("Store");
	gtk_widget_show(storebutton);
	gtk_box_pack_start(GTK_BOX(buttonbox), storebutton, FALSE, FALSE, 0);
	gtk_widget_set_usize(storebutton, 100, 20);
	
	/**********************************************************/
	alertlabel=gtk_label_new("Alerts");
	gtk_widget_show(alertlabel);
	
	vbox2=gtk_vbox_new(FALSE, 0);
	gtk_widget_show(vbox2);
	
	buttonbox2=gtk_hbox_new(TRUE, 0);
	gtk_widget_show(buttonbox2);
	gtk_box_pack_end(GTK_BOX(vbox2), buttonbox2, FALSE, FALSE, 0);
	
	textbox2=gtk_hbox_new(FALSE, 0);
	gtk_widget_show(textbox2);
	gtk_box_pack_end(GTK_BOX(vbox2), textbox2, TRUE, TRUE, 0);

	adj=gtk_adjustment_new(45.0, 0.0, 100.0, 1.0, 5.0, 20.0);
	
	AlertList[SensorNum]=gtk_clist_new_with_titles(4, AlertListLabels);
	gtk_clist_set_vadjustment(GTK_CLIST(AlertList[SensorNum]),GTK_ADJUSTMENT(adj));
	gtk_clist_set_column_auto_resize(GTK_CLIST(AlertList[SensorNum]), 0, TRUE);
	gtk_clist_set_column_auto_resize(GTK_CLIST(AlertList[SensorNum]), 1, TRUE);
	gtk_clist_set_column_auto_resize(GTK_CLIST(AlertList[SensorNum]), 2, TRUE);
	gtk_clist_set_column_auto_resize(GTK_CLIST(AlertList[SensorNum]), 3, TRUE);
	gtk_widget_show(AlertList[SensorNum]);
	gtk_box_pack_start(GTK_BOX(textbox2), AlertList[SensorNum], TRUE, TRUE, 0);
	
	vscrollbar2=gtk_vscrollbar_new(GTK_ADJUSTMENT(adj));
	gtk_widget_show(vscrollbar2);
	gtk_box_pack_end(GTK_BOX(textbox2), vscrollbar2, FALSE, FALSE, 0);
	
	updatebutton2=gtk_button_new_with_label("Fetch");
	gtk_widget_show(updatebutton2);
	gtk_box_pack_start(GTK_BOX(buttonbox2), updatebutton2, FALSE, FALSE, 0);
	gtk_signal_connect(GTK_OBJECT(updatebutton2), "clicked", GTK_SIGNAL_FUNC(GetAlertStart), (void*)SensorNum);
	gtk_widget_set_usize(updatebutton2, 100, 20);
		
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox2, alertlabel);
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox, ruleslabel);

	win = notebook;
	return win;
}

/*****************************************************
* Create the summary window
*****************************************************/
GtkWidget* MakeSummaryWindow(){
	GtkWidget*	summary;
	GtkWidget*	vbox;
	int			i;
	GtkWidget*	TopLabel[MAX_TARGETS];
	GtkWidget*	PacketLabel[MAX_TARGETS];
	GtkWidget*	TCPLabel[MAX_TARGETS];
	GtkWidget*	UDPLabel[MAX_TARGETS];
	GtkWidget*	ICMPLabel[MAX_TARGETS];
	GtkWidget*	ARPLabel[MAX_TARGETS];
	GtkWidget*	IPV6Label[MAX_TARGETS];
	GtkWidget*	IPXLabel[MAX_TARGETS];
	GtkWidget*	OtherLabel[MAX_TARGETS];
	GtkWidget*	AlertLabel[MAX_TARGETS];
	GtkWidget*	LogLabel[MAX_TARGETS];
	GtkWidget*	DropLabel[MAX_TARGETS];
	GtkWidget*	SummaryTable[MAX_TARGETS];
	GtkWidget*	UpdateButton[MAX_TARGETS];
	
	vbox=gtk_hbox_new(TRUE,2);
	gtk_widget_show(vbox);
	
	for (i=0;i<MAX_TARGETS;i++){
	  if (Targets[i].InUse){
		/*make each of the summaries*/
		SummaryTable[i]=gtk_table_new(15, 0, TRUE);
		gtk_widget_show(SummaryTable[i]);
				
		TopLabel[i]=gtk_label_new(Targets[i].Name);
		gtk_widget_show(TopLabel[i]);
		gtk_table_attach(
			GTK_TABLE(SummaryTable[i]),
			TopLabel[i],
			0,2,0,1,0,0,0,0);
		
		PacketLabel[i]=gtk_label_new("Packets:");
		gtk_widget_show(PacketLabel[i]);
		gtk_table_attach(
			GTK_TABLE(SummaryTable[i]),
			PacketLabel[i],
			0,1,1,2,0,0,0,0);
		gtk_label_set_justify(GTK_LABEL(PacketLabel[i]), GTK_JUSTIFY_LEFT);
		
		PacketCount[i]=gtk_label_new("0");
		gtk_widget_show(PacketCount[i]);
		gtk_table_attach(
			GTK_TABLE(SummaryTable[i]),
			PacketCount[i],
			1,2,1,2,0,0,0,0);
		gtk_label_set_justify(GTK_LABEL(PacketCount[i]), GTK_JUSTIFY_LEFT);
		
		TCPLabel[i]=gtk_label_new("TCP Packets:");
		gtk_widget_show(TCPLabel[i]);
		gtk_table_attach(
			GTK_TABLE(SummaryTable[i]),
			TCPLabel[i],
			0,1,2,3,0,0,0,0);	
		gtk_label_set_justify(GTK_LABEL(TCPLabel[i]), GTK_JUSTIFY_LEFT);	
		
		TCPCount[i]=gtk_label_new("0");
		gtk_widget_show(TCPCount[i]);
		gtk_table_attach(
			GTK_TABLE(SummaryTable[i]),
			TCPCount[i],
			1,2,2,3,0,0,0,0);
		gtk_label_set_justify(GTK_LABEL(TCPCount[i]), GTK_JUSTIFY_LEFT);
		
		UDPLabel[i]=gtk_label_new("UDP Packets:");
		gtk_widget_show(UDPLabel[i]);
		gtk_table_attach(
			GTK_TABLE(SummaryTable[i]),
			UDPLabel[i],
			0,1,3,4,0,0,0,0);	
		gtk_label_set_justify(GTK_LABEL(UDPLabel[i]), GTK_JUSTIFY_LEFT);	
		
		UDPCount[i]=gtk_label_new("0");
		gtk_widget_show(UDPCount[i]);
		gtk_table_attach(
			GTK_TABLE(SummaryTable[i]),
			UDPCount[i],
			1,2,3,4,0,0,0,0);
		gtk_label_set_justify(GTK_LABEL(UDPCount[i]), GTK_JUSTIFY_LEFT);
		
		ICMPLabel[i]=gtk_label_new("ICMP Packets:");
		gtk_widget_show(ICMPLabel[i]);
		gtk_table_attach(
			GTK_TABLE(SummaryTable[i]),
			ICMPLabel[i],
			0,1,4,5,0,0,0,0);	
		gtk_label_set_justify(GTK_LABEL(ICMPLabel[i]), GTK_JUSTIFY_LEFT);	
		
		ICMPCount[i]=gtk_label_new("0");
		gtk_widget_show(ICMPCount[i]);
		gtk_table_attach(
			GTK_TABLE(SummaryTable[i]),
			ICMPCount[i],
			1,2,4,5,0,0,0,0);
		gtk_label_set_justify(GTK_LABEL(ICMPCount[i]), GTK_JUSTIFY_LEFT);

		ARPLabel[i]=gtk_label_new("ARP Packets:");
		gtk_widget_show(ARPLabel[i]);
		gtk_table_attach(
			GTK_TABLE(SummaryTable[i]),
			ARPLabel[i],
			0,1,5,6,0,0,0,0);	
		gtk_label_set_justify(GTK_LABEL(ARPLabel[i]), GTK_JUSTIFY_LEFT);	
		
		ARPCount[i]=gtk_label_new("0");
		gtk_widget_show(ARPCount[i]);
		gtk_table_attach(
			GTK_TABLE(SummaryTable[i]),
			ARPCount[i],
			1,2,5,6,0,0,0,0);
		gtk_label_set_justify(GTK_LABEL(ARPCount[i]), GTK_JUSTIFY_LEFT);

		IPV6Label[i]=gtk_label_new("IPV6 Packets:");
		gtk_widget_show(IPV6Label[i]);
		gtk_table_attach(
			GTK_TABLE(SummaryTable[i]),
			IPV6Label[i],
			0,1,6,7,0,0,0,0);	
		gtk_label_set_justify(GTK_LABEL(IPV6Label[i]), GTK_JUSTIFY_LEFT);	
		
		IPV6Count[i]=gtk_label_new("0");
		gtk_widget_show(IPV6Count[i]);
		gtk_table_attach(
			GTK_TABLE(SummaryTable[i]),
			IPV6Count[i],
			1,2,6,7,0,0,0,0);
		gtk_label_set_justify(GTK_LABEL(IPV6Count[i]), GTK_JUSTIFY_LEFT);

		IPXLabel[i]=gtk_label_new("IPX Packets:");
		gtk_widget_show(IPXLabel[i]);
		gtk_table_attach(
			GTK_TABLE(SummaryTable[i]),
			IPXLabel[i],
			0,1,7,8,0,0,0,0);	
		gtk_label_set_justify(GTK_LABEL(IPXLabel[i]), GTK_JUSTIFY_LEFT);	
		
		IPXCount[i]=gtk_label_new("0");
		gtk_widget_show(IPXCount[i]);
		gtk_table_attach(
			GTK_TABLE(SummaryTable[i]),
			IPXCount[i],
			1,2,7,8,0,0,0,0);
		gtk_label_set_justify(GTK_LABEL(IPXCount[i]), GTK_JUSTIFY_LEFT);

		OtherLabel[i]=gtk_label_new("Other Packets:");
		gtk_widget_show(OtherLabel[i]);
		gtk_table_attach(
			GTK_TABLE(SummaryTable[i]),
			OtherLabel[i],
			0,1,8,9,0,0,0,0);	
		gtk_label_set_justify(GTK_LABEL(OtherLabel[i]), GTK_JUSTIFY_LEFT);	
		
		OtherCount[i]=gtk_label_new("0");
		gtk_widget_show(OtherCount[i]);
		gtk_table_attach(
			GTK_TABLE(SummaryTable[i]),
			OtherCount[i],
			1,2,8,9,0,0,0,0);
		gtk_label_set_justify(GTK_LABEL(OtherCount[i]), GTK_JUSTIFY_LEFT);

		AlertLabel[i]=gtk_label_new("Alerts:");
		gtk_widget_show(AlertLabel[i]);
		gtk_table_attach(
			GTK_TABLE(SummaryTable[i]),
			AlertLabel[i],
			0,1,11,12,0,0,0,0);	
		gtk_label_set_justify(GTK_LABEL(AlertLabel[i]), GTK_JUSTIFY_LEFT);	
		
		AlertCount[i]=gtk_label_new("0");
		gtk_widget_show(AlertCount[i]);
		gtk_table_attach(
			GTK_TABLE(SummaryTable[i]),
			AlertCount[i],
			1,2,11,12,0,0,0,0);
		gtk_label_set_justify(GTK_LABEL(AlertCount[i]), GTK_JUSTIFY_LEFT);

		LogLabel[i]=gtk_label_new("Logs:");
		gtk_widget_show(LogLabel[i]);
		gtk_table_attach(
			GTK_TABLE(SummaryTable[i]),
			LogLabel[i],
			0,1,12,13,0,0,0,0);	
		gtk_label_set_justify(GTK_LABEL(LogLabel[i]), GTK_JUSTIFY_LEFT);	
		
		LogCount[i]=gtk_label_new("0");
		gtk_widget_show(LogCount[i]);
		gtk_table_attach(
			GTK_TABLE(SummaryTable[i]),
			LogCount[i],
			1,2,12,13,0,0,0,0);
		gtk_label_set_justify(GTK_LABEL(LogCount[i]), GTK_JUSTIFY_LEFT);
		
		DropLabel[i]=gtk_label_new("Drops:");
		gtk_widget_show(DropLabel[i]);
		gtk_table_attach(
			GTK_TABLE(SummaryTable[i]),
			DropLabel[i],
			0,1,13,14,0,0,0,0);	
		gtk_label_set_justify(GTK_LABEL(DropLabel[i]), GTK_JUSTIFY_LEFT);	
		
		DropCount[i]=gtk_label_new("0");
		gtk_widget_show(DropCount[i]);
		gtk_table_attach(
			GTK_TABLE(SummaryTable[i]),
			DropCount[i],
			1,2,13,14,0,0,0,0);
		gtk_label_set_justify(GTK_LABEL(DropCount[i]), GTK_JUSTIFY_LEFT);

		UpdateButton[i]=gtk_button_new_with_label("Update");
		gtk_widget_show(UpdateButton[i]);
		gtk_table_attach(
			GTK_TABLE(SummaryTable[i]),
			UpdateButton[i],
			0,2,14,15,GTK_FILL,GTK_FILL,0,0);
		gtk_signal_connect(GTK_OBJECT(UpdateButton[i]), "clicked", GTK_SIGNAL_FUNC(SummaryUpdateClicked), (void*)i);

		
		gtk_box_pack_start(GTK_BOX(vbox), SummaryTable[i], FALSE, FALSE, 1);
	  }	
	}
	
	summary=vbox;
	return summary;
}

/******************************************************
* Create the main window
*******************************************************/
GtkWidget* MakeMainWindow(){
	GtkWidget*	window;
	GtkWidget*	notebook;
	GtkWidget* 	dummy[MAX_TARGETS];
	GtkWidget*  label[MAX_TARGETS];
	GtkWidget* 	summary;
	int			i;
		
	window=gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_widget_realize(window);
	gtk_signal_connect(GTK_OBJECT(window), "delete_event", GTK_SIGNAL_FUNC(gtk_main_quit), NULL);
	gtk_signal_connect(GTK_OBJECT(window), "destroy", GTK_SIGNAL_FUNC(gtk_main_quit), NULL);
	gtk_widget_set_usize(GTK_WIDGET(window),640, 480);

	summary=MakeSummaryWindow();

	notebook=gtk_notebook_new();
	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(notebook), GTK_POS_TOP);
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), summary, xpm_label_box(window, "pics/summary-small.xpm","summary"));
	
	for (i=0;i<MAX_TARGETS;i++){
	  if (Targets[i].InUse){
		dummy[i]=MakeSensorWindow(i);
		label[i]=gtk_label_new(Targets[i].Name);
		gtk_widget_show(dummy[i]);
		gtk_widget_show(label[i]);
		gtk_notebook_append_page(GTK_NOTEBOOK(notebook), dummy[i], xpm_label_box(window, "pics/host-small.xpm",Targets[i].Name));
	  }	
	}
	
	gtk_widget_show(notebook);
	gtk_container_add(GTK_CONTAINER(window), notebook);
	
	
	return window;
}


/********************************************************************
* Gets called when the stat packet is ready
********************************************************************/
void UpdateStatusEnd(int success, void* data){
	JobStat*			j;
	unsigned long long 	total;
	char				Buff[4096];
	int					SensorNum;
	
	
	/*if we didn't get it, bail*/
	if (!success) return;
	
	j=(JobStat*)data;
	SensorNum=(int)j->User;
	
	total=j->Stat->TCPCount + j->Stat->UDPCount + j->Stat->ICMPCount + j->Stat->ARPCount 
		+ j->Stat->IPV6Count + j->Stat->IPXCount + j->Stat->OtherCount;
	if (total==0) return;
	sprintf(Buff, "%llu(%i%%)", total, (total/total)*100);
	gtk_label_set_text(GTK_LABEL(PacketCount[SensorNum]), Buff);
	sprintf(Buff, "%i(%i%%)", j->Stat->AlertCount, j->Stat->AlertCount*100/total);
	gtk_label_set_text(GTK_LABEL(AlertCount[SensorNum]), Buff);
	sprintf(Buff, "%i(%i%%)", j->Stat->DropCount, j->Stat->DropCount*100/total);
	gtk_label_set_text(GTK_LABEL(DropCount[SensorNum]), Buff);
	sprintf(Buff, "%i(%i%%)", j->Stat->LogCount, j->Stat->LogCount*100/total);
	gtk_label_set_text(GTK_LABEL(LogCount[SensorNum]), Buff);
	sprintf(Buff, "%i(%i%%)", j->Stat->TCPCount, j->Stat->TCPCount*100/total);
	gtk_label_set_text(GTK_LABEL(TCPCount[SensorNum]), Buff);
	sprintf(Buff, "%i(%i%%)", j->Stat->UDPCount, j->Stat->UDPCount*100/total);
	gtk_label_set_text(GTK_LABEL(UDPCount[SensorNum]), Buff);
	sprintf(Buff, "%i(%i%%)", j->Stat->ICMPCount, j->Stat->ICMPCount*100/total);
	gtk_label_set_text(GTK_LABEL(ICMPCount[SensorNum]), Buff);
	sprintf(Buff, "%i(%i%%)", j->Stat->ARPCount, j->Stat->ARPCount*100/total);
	gtk_label_set_text(GTK_LABEL(ARPCount[SensorNum]), Buff);
	sprintf(Buff, "%i(%i%%)", j->Stat->IPV6Count, j->Stat->IPV6Count*100/total);
	gtk_label_set_text(GTK_LABEL(IPV6Count[SensorNum]), Buff);
	sprintf(Buff, "%i(%i%%)", j->Stat->IPXCount, j->Stat->IPXCount*100/total);
	gtk_label_set_text(GTK_LABEL(IPXCount[SensorNum]), Buff);
	sprintf(Buff, "%i(%i%%)", j->Stat->OtherCount, j->Stat->OtherCount*100/total);
	gtk_label_set_text(GTK_LABEL(OtherCount[SensorNum]), Buff);
	
	free(j->Stat);
	free(data);
}

/********************************************************************
* Start the Update 
********************************************************************/
void UpdateStatusStart(int SensorNum){
	ControlPacketStatReply*	st;
	
	st=(ControlPacketStatReply*)calloc(sizeof(ControlPacketStatReply),1);
	hn_get_stat(Targets[SensorNum].IP,0, st,UpdateStatusEnd,(void*)SensorNum);
}


/**********************************************************************
* Called when network stuff is ready to be processed
**********************************************************************/
void DoNetwork(gpointer data, gint source, GdkInputCondition condition){
	hn_read_sock();
}

/**********************************************************************
* Called at intervals to resend packets
**********************************************************************/
int DoIdleNetwork(gpointer data){
	hm_resend_all();
	return TRUE;
}

/********************************************************************
* The one and only main
********************************************************************/
int main(int argc, char**argv){
	struct in_addr 	ip;
	char			Buff[65535];
	int				i;
	int				BytesReturned;
	GtkWidget*		MainWindow;
	
	
	gtk_init(&argc, &argv);
	hm_init("control.conf");
	gdk_input_add(GlobalFD, GDK_INPUT_READ, DoNetwork, NULL);
	gtk_timeout_add(200, DoIdleNetwork, NULL);
	
	MainWindow=MakeMainWindow();
	gtk_widget_show(MainWindow);
		
	/*inet_aton("10.1.1.1",&ip);
	
	
	if (!chogwash_ping(ip, 3)){
		printf("Hogwash box isn't alive\n");
		return 0;
	}

	for (i=0;i<100000;i=i+100){
		bzero(Buff,101);
		switch (chogwash_get_buff_chunk(ip, 3, STOCK_BUFFER_RULES, i, 100, Buff, &BytesReturned)){
		case BUFFER_FLAG_OK:
			printf("%s",Buff);
			break;
		case BUFFER_FLAG_TIMED_OUT:
			i=i-100;
			break;	
		default:
			i=999999;
		}	
	}*/
	
	gtk_main();
	hm_close();
}
